package com.miyzh.rabbitMQ;

import com.miyzh.config.RabbitConfig;
import com.miyzh.entity.UploadExceptionLog;
import com.miyzh.service.UploadService;
import com.miyzh.utils.DesUtil;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * @Description: 消费者
 * @Author: lyt
 * @Date: 2019/6/21 11:37
 */
@Component
public class MsgReceiver {

    @Autowired
    private UploadService uploadService;

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @RabbitListener(queues = "preSettle")
    public void preSettle(String content, Message message, Channel channel) throws IOException {
        try {
//            int a = 1 / 0;
            System.out.println("消费消息：" + content);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), true);
            UploadExceptionLog uploadExceptionLog = new UploadExceptionLog();
            uploadExceptionLog.setUploadInfo(DesUtil.decrypt(content));
            uploadService.upload(uploadExceptionLog);
            System.out.println("消费消息确认" + message.getMessageProperties().getConsumerQueue() + "，接收到了回调方法");
        } catch (Exception e) {
            //重新回到队列
            channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
            System.out.println("尝试重发：" + message.getMessageProperties().getConsumerQueue());
            //requeue =true 重回队列，false 丢弃
//            channel.basicReject(message.getMessageProperties().getDeliveryTag(), false);
            // TODO 该消息已经导致异常，重发无意义，自己实现补偿机制


        }
    }

    @RabbitListener(queues = "confirmSettle")
    public void confirmSettle(String content, Message message, Channel channel) throws IOException {
        try {
            System.out.println("消费消息：" + content);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), true);
            UploadExceptionLog uploadExceptionLog = new UploadExceptionLog();
            uploadExceptionLog.setUploadInfo(DesUtil.decrypt(content));
            uploadService.upload(uploadExceptionLog);
            System.out.println("消费消息确认" + message.getMessageProperties().getConsumerQueue() + "，接收到了回调方法");
        } catch (Exception e) {
            //重新回到队列
            channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
            System.out.println("尝试重发：" + message.getMessageProperties().getConsumerQueue());
            //requeue =true 重回队列，false 丢弃
//            channel.basicReject(message.getMessageProperties().getDeliveryTag(), false);
            // TODO 该消息已经导致异常，重发无意义，自己实现补偿机制


        }
    }
}
