package com.miyzh.service;

import com.alibaba.fastjson.JSONObject;
import com.miyzh.entity.UploadExceptionLog;
import com.miyzh.utils.DesUtil;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * description: UploadService
 * date: 2020/6/18 15:48
 * author: SZQ
 */
@Service
public class UploadService {

    @Autowired
    private WsClientService wsClientService;
    @Autowired
    private UploadExceptionLogService uploadExceptionLogService;
    @Value("${scheduling.limit}")
    private int limit;
    @Value("${ws.webUrl}")
    private String webUrl;

    private final Logger logger = LoggerFactory.getLogger(this.getClass());


    public String upload(UploadExceptionLog uploadExceptionLog) {
        try {
            Document doc = DocumentHelper.parseText(uploadExceptionLog.getUploadInfo());
//            Document doc = DocumentHelper.parseText(uploadExceptionLog.getUploadInfo());
            if (StringUtils.isBlank(uploadExceptionLog.getTradeCode())) {
                if (StringUtils.isBlank(doc.getRootElement().element("TradeCode").getTextTrim())) {
                    logger.error("交易码不能为空");
                    return "交易码不能为空";
                }
                uploadExceptionLog.setTradeCode(doc.getRootElement().element("TradeCode").getTextTrim());
            }
            if (StringUtils.isBlank(uploadExceptionLog.getSerialNumber())) {
                if (StringUtils.isBlank(doc.getRootElement().element("SERIAL_NUMBER").getTextTrim())) {
                    return "业务流水号不能为空";
                }
                uploadExceptionLog.setSerialNumber(doc.getRootElement().element("SERIAL_NUMBER").getTextTrim());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return "系统异常";
        }
        return runservice(uploadExceptionLog);
    };

    // 两种情况 1:第一次调用 2:第二次调用,第二次调用成功标记删除
    public String runservice(UploadExceptionLog uploadExceptionLog) {
        try{
            System.out.println("GetIn");
            String methodName = "runservice";
            String param = uploadExceptionLog.getUploadInfo();
            String resultCode = JSONObject.parseObject(wsClientService.callWebSV(webUrl, methodName, param)).get("resultCode").toString();
            // 第二次请求
            if (null != uploadExceptionLog.getId() && uploadExceptionLog.getId() > 0) {
                // 如果有返回证明请求正常,标记删除该记录并记录状态,如果再次失败证明请求参数异常,更新异常信息
                if (StringUtils.isNotBlank(resultCode)) {
                    uploadExceptionLog.setIsDel(0);
                    // 成功
                    if ("0".equals(resultCode)) {
                        uploadExceptionLog.setStatus(1);
                    } else {
                        uploadExceptionLog.setStatus(3);
                        uploadExceptionLog.setErrorMsg(JSONObject.parseObject(wsClientService.callWebSV(webUrl, methodName, param))
                                .get("resultContent").toString());
                    }
                    uploadExceptionLogService.update(uploadExceptionLog);
                }
            } else {
                if (StringUtils.isNotBlank(resultCode) && "1".equals(resultCode)) {
                    uploadExceptionLog.setIsDel(0);
                    uploadExceptionLog.setStatus(3);
                    uploadExceptionLog.setErrorMsg(JSONObject.parseObject(wsClientService.callWebSV(webUrl, methodName, param))
                            .get("resultContent").toString());
                    uploadExceptionLogService.inster(uploadExceptionLog);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            uploadExceptionLog.setErrorMsg(e.getCause().getMessage());
            uploadExceptionLog.setIsDel(1);
            uploadExceptionLog.setStatus(0);
            if (null != uploadExceptionLog.getId() && uploadExceptionLog.getId() > 0) {
                // 修改
                uploadExceptionLogService.update(uploadExceptionLog);
            } else {
                // 新增异常记录
                uploadExceptionLogService.inster(uploadExceptionLog);
            }
        }
        return null;
    }

    public void retryUpload() {
        List<UploadExceptionLog> list = uploadExceptionLogService.listUploadExceptionLogLimit(limit);
        System.out.println("开始重传异常记录");
        if (null != list && list.size() > 0) {
            list.stream().forEach(item -> {
                upload(item);
            });
        }
    }

    public static void main(String[] args) {
    }
}
